"""Tests for ClientCookie._ClientCookie."""

import urllib2, re, os, string, StringIO, mimetools, time
from time import localtime
from unittest import TestCase

from ClientCookie._Util import startswith

#ClientCookie.CLIENTCOOKIE_DEBUG = 1

class FakeResponse:
    def __init__(self, headers=[], url=None):
        """
        headers: list of RFC822-style 'Key: value' strings
        """
        f = StringIO.StringIO(string.join(headers, "\n"))
        self._headers = mimetools.Message(f)
        self._url = url
    def info(self): return self._headers
    def url(): return self._url

def interact(c, url, *args):
    return _interact(c, url, args, netscape=0)

def interact_netscape(c, url, *args):
    return _interact(c, url, args, netscape=1)

def _interact(c, url, args, netscape):
    req = urllib2.Request(url)
    c.add_cookie_header(req)
    cookie = req.headers.get("Cookie", "")
    headers = []
    for arg in args:
        if netscape:
            headers.append("Set-Cookie: %s" % (arg,))
        else:
            headers.append("Set-Cookie2: %s" % (arg,))
    res = FakeResponse(headers, url)
    c.extract_cookies(res, req)
    return cookie


class CookieTests(TestCase):
    # XXX
    # Set-Cookie2 without version attribute is rejected.
    # Netscape cookies can be sent back to any port.
    # Set-Cookie with negative max age.
    # If turn RFC 2965 handling off, Set-Cookie2 cookies should not clobber
    #  Set-Cookie cookies.
    # Proper testing of when RFC 2965 clobbers Netscape.
    # RFC 2965 acceptance and returning rules!
    # Netscape peculiarities list from ronald@innovation.ch:
## - Quoting: only quotes around the expires value are recognized as such
##   (and yes, some folks quote the expires value); quotes around any other
##   value are treated as part of the value.
## - White space: white space around names and values is ignored
## - Default path: if no path parameter is given, the path defaults to the
##   path in the request-uri up to, but not including, the last '/'. Note
##   that this is entirely different from what the spec says.
## - Commas and other delimiters: Netscape just parses until the next ';'.
##   This means it will allow commas etc inside values (and yes, both
##   commas and equals are commonly appear in the cookie value). This also
##   means that if you fold multiple Set-Cookie header fields into one,
##   comma-separated list, it'll be a headache to parse (at least my head
##   starts hurting everytime I think of that code).
## - Expires: You'll get all sorts of date formats in the expires,
##   including emtpy expires attributes ("expires="). Be as flexible as you
##   can, and certainly don't expect the weekday to be there; if you can't
##   parse it, just ignore it and pretend it's a session cookie.
## - Domain-matching: Netscape uses the 2-dot rule for _all_ domains, not
##   just the 7 special TLD's listed in their spec. And folks rely on
##   that...

    def test_default_path(self):
        from ClientCookie import Cookies

        # RFC 2965

        c = Cookies()
        interact(c, "http://www.acme.com/", 'spam="bar"; Version="1"')
        assert c.cookies["www.acme.com"].has_key("/")

        c = Cookies()
        interact(c, "http://www.acme.com/blah", 'eggs="bar"; Version="1"')
        assert c.cookies["www.acme.com"].has_key("/")
     
        c = Cookies()
        interact(c, "http://www.acme.com/blah/rhubarb",
                 'eggs="bar"; Version="1"')
        assert c.cookies["www.acme.com"].has_key("/blah/")

        c = Cookies()
        interact(c, "http://www.acme.com/blah/rhubarb/",
                 'eggs="bar"; Version="1"')
        assert c.cookies["www.acme.com"].has_key("/blah/rhubarb/")

        # Netscape

        c = Cookies()
        interact_netscape(c, "http://www.acme.com/", 'spam="bar"')
        assert c.cookies["www.acme.com"].has_key("/")

        c = Cookies()
        interact_netscape(c, "http://www.acme.com/blah", 'eggs="bar"')
        assert c.cookies["www.acme.com"].has_key("/")
     
        c = Cookies()
        interact_netscape(c, "http://www.acme.com/blah/rhubarb", 'eggs="bar"')
        assert c.cookies["www.acme.com"].has_key("/blah")

        c = Cookies()
        interact_netscape(c, "http://www.acme.com/blah/rhubarb/", 'eggs="bar"')
        assert c.cookies["www.acme.com"].has_key("/blah/rhubarb")

    def test_request_host(self):
        from urllib2 import Request
        from ClientCookie._ClientCookie import request_host
        # this request is illegal (RFC2616, 14.2.3)
        req = Request("http://1.1.1.1/",
                      headers={"Host": "www.acme.com:80"})
        # libwww-perl wants this response, but that seems wrong (RFC 2616,
        # section 5.2, point 1., and RFC 2965 section 1, paragraph 3)
        #assert request_host(req) == "www.acme.com"
        assert request_host(req) == "1.1.1.1"
        req = Request("http://www.acme.com/",
                      headers={"Host": "irrelevant.com"})
        assert request_host(req) == "www.acme.com"
        # not actually sure this one is valid Request object, so maybe should
        # remove test for no host in url in request_host function?
        req = Request("/resource.html",
                      headers={"Host": "www.acme.com"})
        assert request_host(req) == "www.acme.com"

    def test_is_HDN(self):
        from ClientCookie._ClientCookie import is_HDN
        assert is_HDN("foo.bar.com")
        assert is_HDN("1foo2.3bar4.5com")
        assert not is_HDN("192.168.1.1")
        assert not is_HDN("")
        assert not is_HDN(".")
        assert not is_HDN(".foo.bar.com")
        assert not is_HDN("..foo")
        assert not is_HDN("foo.")

    def test_domain_match(self):
        from ClientCookie._ClientCookie import domain_match, liberal_domain_match
        assert domain_match("192.168.1.1", "192.168.1.1")
        assert not domain_match("192.168.1.1", ".168.1.1")
        assert domain_match("x.y.com", "x.Y.com")
        assert domain_match("x.y.com", ".Y.com")
        assert not domain_match("x.y.com", "Y.com")
        assert domain_match("a.b.c.com", ".c.com")
        assert not domain_match(".c.com", "a.b.c.com")
        assert domain_match("example.local", ".local")
        assert not domain_match("blah.blah", "")
        assert not domain_match("", ".rhubarb.rhubarb")
        assert domain_match("", "")

        assert liberal_domain_match("x.y.com", "x.Y.com")
        assert liberal_domain_match("x.y.com", ".Y.com")
        assert liberal_domain_match("x.y.com", "Y.com")
        assert liberal_domain_match("x.y.com", ".com")
        assert liberal_domain_match("x.y.com", "com")
        assert liberal_domain_match("x.y.com", "m")
        assert liberal_domain_match("x.y.com", "")
        assert liberal_domain_match("192.168.1.1", "192.168.1.1")
        # not both HDNs, so must string-compare equal to match
        assert not liberal_domain_match("192.168.1.1", ".168.1.1")
        # empty string is a special case
        assert liberal_domain_match("192.168.1.1", "")

    def test_wrong_domain(self):
        """Cookies whose ERH does not domain-match the domain are rejected.

        ERH = effective request-host.

        """
        # XXX far from complete
        from ClientCookie import Cookies
        c = Cookies()
        interact(c, "http://www.nasty.com/", 'foo=bar; domain=friendly.org; Version="1"')
        assert len(c) == 0

    def test_two_component_domain(self):
        # XXX Not quite covered:
        # Check V1 explicit Domain=acme.com gets sent back to *.acme.com, but
        #  not acme.com (I think...!) AND missing Domain attr. from acme.com server
        #  results in cookie sent back ONLY to acme.com.
        from ClientCookie import Cookies
        c = Cookies()

        # two-component domain is OK
        interact(c, "http://foo.net/", 'foo=bar; Version="1"')
        assert len(c) == 1
        assert interact(c, "http://foo.net/") == "$Version=1; foo=bar"

        # unless domain is given explicitly, because then it must be
        # rewritten to start with a dot: foo.net --> .foo.net
        interact(c, "http://foo.net/",
                 'spam=eggs; domain=foo.net; Version="1"')
        assert len(c) == 1
        assert interact(c, "http://foo.net/") == "$Version=1; foo=bar"

        # single-component domain is too general
        interact(c, "http://foo.net/",
                 'blah=rhubarb; domain=.net; Version="1"')
        assert len(c) == 1

        # RFC 2965 doesn't require blocking this
        interact(c, "http://foo.co.uk/",
                 'nasty=trick; domain=.co.uk; Version="1"')
        assert len(c) == 2

        c = Cookies()

        interact_netscape(c, "http://foo.net/", 'foo=bar')
        assert len(c) == 1
        assert interact_netscape(c, "http://foo.net/") == "foo=bar"

        interact_netscape(c, "http://foo.net/", 'spam=eggs; domain=foo.net')
        assert len(c) == 2
        assert interact_netscape(c, "http://foo.net/") == "foo=bar; spam=eggs"

##         # Netscape protocol doesn't allow non-special top level domains (such
##         # as co.uk) in the domain attribute unless there are at least three
##         # dots in it.
        # Oh yes it does!  Real implementations don't check this, and real
        # cookies (of course) rely on that behaviour.
        interact_netscape(c, "http://foo.co.uk", 'nasty=trick; domain=.co.uk')
##         assert len(c) == 2
        assert len(c) == 3

    def test_domain_block(self):
        from ClientCookie import Cookies

        year_plus_one = localtime(time.time())[0] + 1

        headers = []
        c = Cookies(blocked_domains=["acme.com"])

        req = urllib2.Request("http://www.acme.com/")
        headers.append(
            "Set-Cookie: CUSTOMER=WILE_E_COYOTE; path=/ ; "
            "expires=Wednesday, 09-Nov-%d 23:12:40 GMT" % year_plus_one)
        res = FakeResponse(headers, "http://www.acme.com/")
        c.extract_cookies(res, req)
        assert len(c) == 0

        req = urllib2.Request("http://www.roadrunner.net/")
        c.add_cookie_header(req)

        assert not (req.headers.has_key("Cookie") or
                    req.headers.has_key("Cookie2"))

        res = FakeResponse(headers, "http://www.roadrunner.net/")
        c.extract_cookies(res, req)
        req = urllib2.Request("http://www.roadrunner.net/")
        c.add_cookie_header(req)

        assert (req.headers["Cookie"] == "CUSTOMER=WILE_E_COYOTE" and
                req.headers["Cookie2"] == '$Version="1"')

        assert len(c) == 1

    def test_secure(self):
        # XXX yuck, too much cut-n-paste in here!
        from ClientCookie import Cookies
        c = Cookies()
        interact(c, "http://www.acme.com/", "foo1=bar; Version=1")
        interact(c, "http://www.acme.com/", "foo2=bar; Version=1; secure ")
        assert not c.cookies["www.acme.com"]["/"]["foo1"][4], \
               "non-secure cookie registered secure"
        assert c.cookies["www.acme.com"]["/"]["foo2"][4], \
            "secure cookie registered non-secure"

        c = Cookies(netscape_only=1)
        interact_netscape(c, "http://www.acme.com/", "foo1=bar")
        interact_netscape(c, "http://www.acme.com/", "foo2=bar; secure ")
        assert not c.cookies["www.acme.com"]["/"]["foo1"][4], \
               "non-secure cookie registered secure"
        assert c.cookies["www.acme.com"]["/"]["foo2"][4], \
            "secure cookie registered non-secure"

        c = Cookies()
        interact(c, "http://www.acme.com/", "foo1=bar; Version=1")
        interact(c, "http://www.acme.com/", "foo2=bar; Version=1; secure")
        assert not c.cookies["www.acme.com"]["/"]["foo1"][4], \
            "non-secure cookie registered secure"
        assert c.cookies["www.acme.com"]["/"]["foo2"][4], \
            "secure cookie registered non-secure"

        c = Cookies(netscape_only=1)
        interact_netscape(c, "http://www.acme.com/", "foo1=bar")
        interact_netscape(c, "http://www.acme.com/", "foo2=bar; secure")
        assert not c.cookies["www.acme.com"]["/"]["foo1"][4], \
            "non-secure cookie registered secure"
        assert c.cookies["www.acme.com"]["/"]["foo2"][4], \
               "secure cookie registered non-secure"

    def test_quote_cookie_value(self):
        from ClientCookie import Cookies
        c = Cookies()
        interact(c, "http://www.acme.com/", r'foo=\b"a"r; Version=1')
        req = urllib2.Request("http://www.acme.com/")
        c.add_cookie_header(req)
        assert req.headers["Cookie"] == r'$Version=1; foo=\\b\"a\"r'

    def test_missing_final_slash(self):
        """Missing slash from request URL's abs_path should be assumed present.
        """
        from ClientCookie import Cookies
        url = "http://www.acme.com"
        c = Cookies()
        interact(c, url, "foo=bar; Version=1")
        req = urllib2.Request(url)
        assert len(c) == 1
        c.add_cookie_header(req)
        assert req.headers.has_key("Cookie")


class LWPCookieTests(TestCase):
    def test_1(self):
        from ClientCookie import Cookies

        #ClientCookie.CLIENTCOOKIE_DEBUG = 1

        #-------------------------------------------------------------------
        # First we check that it works for the original example at
        # http://www.netscape.com/newsref/std/cookie_spec.html

        # Client requests a document, and receives in the response:
        # 
        #       Set-Cookie: CUSTOMER=WILE_E_COYOTE; path=/; expires=Wednesday, 09-Nov-99 23:12:40 GMT
        # 
        # When client requests a URL in path "/" on this server, it sends:
        # 
        #       Cookie: CUSTOMER=WILE_E_COYOTE
        # 
        # Client requests a document, and receives in the response:
        # 
        #       Set-Cookie: PART_NUMBER=ROCKET_LAUNCHER_0001; path=/
        # 
        # When client requests a URL in path "/" on this server, it sends:
        # 
        #       Cookie: CUSTOMER=WILE_E_COYOTE; PART_NUMBER=ROCKET_LAUNCHER_0001
        # 
        # Client receives:
        # 
        #       Set-Cookie: SHIPPING=FEDEX; path=/fo
        # 
        # When client requests a URL in path "/" on this server, it sends:
        # 
        #       Cookie: CUSTOMER=WILE_E_COYOTE; PART_NUMBER=ROCKET_LAUNCHER_0001
        # 
        # When client requests a URL in path "/foo" on this server, it sends:
        # 
        #       Cookie: CUSTOMER=WILE_E_COYOTE; PART_NUMBER=ROCKET_LAUNCHER_0001; SHIPPING=FEDEX
        # 
        # The last Cookie is buggy, because both specifications say that the
        # most specific cookie must be sent first.  SHIPPING=FEDEX is the
        # most specific and should thus be first.

        year_plus_one = localtime(time.time())[0] + 1

        headers = []

        c = Cookies()

        #req = urllib2.Request("http://1.1.1.1/",
        #                      headers={"Host": "www.acme.com:80"})
        req = urllib2.Request("http://www.acme.com:80/",
                              headers={"Host": "www.acme.com:80"})

        # urllib2 returns mimetools.Message if 200 OK, and presumably the same in other
        #  cases too: you don't get to know the status code AFAICT -- see Python 2.1
        #  urllib2.py, line 816.
        headers.append(
            "Set-Cookie: CUSTOMER=WILE_E_COYOTE; path=/ ; "
            "expires=Wednesday, 09-Nov-%d 23:12:40 GMT" % (year_plus_one,))
        res = FakeResponse(headers, "http://www.acme.com/")
        c.extract_cookies(res, req)

        req = urllib2.Request("http://www.acme.com/")
        c.add_cookie_header(req)

        assert (req.headers["Cookie"] == "CUSTOMER=WILE_E_COYOTE" and
                req.headers["Cookie2"] == '$Version="1"')

        headers.append("Set-Cookie: PART_NUMBER=ROCKET_LAUNCHER_0001; path=/")
        res = FakeResponse(headers, "http://www.acme.com/")
        c.extract_cookies(res, req)

        req = urllib2.Request("http://www.acme.com/foo/bar")
        c.add_cookie_header(req)

        h = req.headers["Cookie"]
        assert (string.find(h, "PART_NUMBER=ROCKET_LAUNCHER_0001") != -1 and
                string.find(h, "CUSTOMER=WILE_E_COYOTE") != -1)


        headers.append('Set-Cookie: SHIPPING=FEDEX; path=/foo')
        res = FakeResponse(headers, "http://www.acme.com")
        c.extract_cookies(res, req)

        req = urllib2.Request("http://www.acme.com/")
        c.add_cookie_header(req)

        h = req.headers["Cookie"]
        assert (string.find(h, "PART_NUMBER=ROCKET_LAUNCHER_0001") != -1 and
                string.find(h, "CUSTOMER=WILE_E_COYOTE") != -1 and
                not string.find(h, "SHIPPING=FEDEX") != -1)


        req = urllib2.Request("http://www.acme.com/foo/")
        c.add_cookie_header(req)

        h = req.headers["Cookie"]
        assert (string.find(h, "PART_NUMBER=ROCKET_LAUNCHER_0001") != -1 and
                string.find(h, "CUSTOMER=WILE_E_COYOTE") != -1 and
                startswith(h, "SHIPPING=FEDEX;"))


    def test_2(self):
        from ClientCookie import Cookies

        # Second Example transaction sequence:
        # 
        # Assume all mappings from above have been cleared.
        # 
        # Client receives:
        # 
        #       Set-Cookie: PART_NUMBER=ROCKET_LAUNCHER_0001; path=/
        # 
        # When client requests a URL in path "/" on this server, it sends:
        # 
        #       Cookie: PART_NUMBER=ROCKET_LAUNCHER_0001
        # 
        # Client receives:
        # 
        #       Set-Cookie: PART_NUMBER=RIDING_ROCKET_0023; path=/ammo
        # 
        # When client requests a URL in path "/ammo" on this server, it sends:
        # 
        #       Cookie: PART_NUMBER=RIDING_ROCKET_0023; PART_NUMBER=ROCKET_LAUNCHER_0001
        # 
        #       NOTE: There are two name/value pairs named "PART_NUMBER" due to
        #       the inheritance of the "/" mapping in addition to the "/ammo" mapping. 

        c = Cookies()
        headers = []

        req = urllib2.Request("http://www.acme.com/")
        headers.append("Set-Cookie: PART_NUMBER=ROCKET_LAUNCHER_0001; path=/")
        res = FakeResponse(headers, "http://www.acme.com/")

        c.extract_cookies(res, req)

        req = urllib2.Request("http://www.acme.com/")
        c.add_cookie_header(req)

        assert (req.headers["Cookie"] == "PART_NUMBER=ROCKET_LAUNCHER_0001")

        headers.append("Set-Cookie: PART_NUMBER=RIDING_ROCKET_0023; path=/ammo")
        res = FakeResponse(headers, "http://www.acme.com/")
        c.extract_cookies(res, req)

        req = urllib2.Request("http://www.acme.com/ammo")
        c.add_cookie_header(req)

        assert re.search(r"PART_NUMBER=RIDING_ROCKET_0023;\s*"
                         "PART_NUMBER=ROCKET_LAUNCHER_0001", req.headers["Cookie"])


    # XXX I don't know when you'd ever get a response with no URL.
##     def test_3(self):
##         #-------------------------------------------------------------------
##         # When there are no "Set-Cookie" header, then even responses
##         # without any request URLs connected should be allowed.

##         c = Cookies()
##         c.extract_cookies(FakeResponse(), ??)
##         assert not c


    def test_4(self):
        from ClientCookie import Cookies
        #-------------------------------------------------------------------
        # Then we test with the examples from draft-ietf-http-state-man-mec-03.txt
        #
        # 5.  EXAMPLES

        c = Cookies()

        # 
        # 5.1  Example 1
        # 
        # Most detail of request and response headers has been omitted.  Assume
        # the user agent has no stored cookies.
        # 
        #   1.  User Agent -> Server
        # 
        #       POST /acme/login HTTP/1.1
        #       [form data]
        # 
        #       User identifies self via a form.
        # 
        #   2.  Server -> User Agent
        # 
        #       HTTP/1.1 200 OK
        #       Set-Cookie2: Customer="WILE_E_COYOTE"; Version="1"; Path="/acme"
        # 
        #       Cookie reflects user's identity.

        cookie = interact(c, 'http://www.acme.com/acme/login',
                          'Customer="WILE_E_COYOTE"; Version="1"; Path="/acme"')
        assert not cookie

        # 
        #   3.  User Agent -> Server
        # 
        #       POST /acme/pickitem HTTP/1.1
        #       Cookie: $Version="1"; Customer="WILE_E_COYOTE"; $Path="/acme"
        #       [form data]
        # 
        #       User selects an item for ``shopping basket.''
        # 
        #   4.  Server -> User Agent
        # 
        #       HTTP/1.1 200 OK
        #       Set-Cookie2: Part_Number="Rocket_Launcher_0001"; Version="1";
        #               Path="/acme"
        # 
        #       Shopping basket contains an item.

        cookie = interact(c, 'http://www.acme.com/acme/pickitem',
                          'Part_Number="Rocket_Launcher_0001"; '
                          'Version="1"; Path="/acme"');
        assert re.search(
            r'^\$Version="?1"?; Customer="?WILE_E_COYOTE"?; \$Path="/acme"$', cookie)

        # 
        #   5.  User Agent -> Server
        # 
        #       POST /acme/shipping HTTP/1.1
        #       Cookie: $Version="1";
        #               Customer="WILE_E_COYOTE"; $Path="/acme";
        #               Part_Number="Rocket_Launcher_0001"; $Path="/acme"
        #       [form data]
        # 
        #       User selects shipping method from form.
        # 
        #   6.  Server -> User Agent
        # 
        #       HTTP/1.1 200 OK
        #       Set-Cookie2: Shipping="FedEx"; Version="1"; Path="/acme"
        # 
        #       New cookie reflects shipping method.

        cookie = interact(c, "http://www.acme.com/acme/shipping",
                          'Shipping="FedEx"; Version="1"; Path="/acme"')

        assert (re.search(r'^\$Version="?1"?;', cookie) and
                re.search(r'Part_Number="?Rocket_Launcher_0001"?;'
                          '\s*\$Path="\/acme"', cookie) and
                re.search(r'Customer="?WILE_E_COYOTE"?;\s*\$Path="\/acme"', cookie))

        # 
        #   7.  User Agent -> Server
        # 
        #       POST /acme/process HTTP/1.1
        #       Cookie: $Version="1";
        #               Customer="WILE_E_COYOTE"; $Path="/acme";
        #               Part_Number="Rocket_Launcher_0001"; $Path="/acme";
        #               Shipping="FedEx"; $Path="/acme"
        #       [form data]
        # 
        #       User chooses to process order.
        # 
        #   8.  Server -> User Agent
        # 
        #       HTTP/1.1 200 OK
        # 
        #       Transaction is complete.

        cookie = interact(c, "http://www.acme.com/acme/process")
        assert (re.search(r'Shipping="?FedEx"?;\s*\$Path="\/acme"', cookie) and
                string.find(cookie, "WILE_E_COYOTE") != -1)

        # 
        # The user agent makes a series of requests on the origin server, after
        # each of which it receives a new cookie.  All the cookies have the same
        # Path attribute and (default) domain.  Because the request URLs all have
        # /acme as a prefix, and that matches the Path attribute, each request
        # contains all the cookies received so far.


    def test_5(self):
        from ClientCookie import Cookies

        # 5.2  Example 2
        # 
        # This example illustrates the effect of the Path attribute.  All detail
        # of request and response headers has been omitted.  Assume the user agent
        # has no stored cookies.

        c = Cookies()

        # Imagine the user agent has received, in response to earlier requests,
        # the response headers
        # 
        # Set-Cookie2: Part_Number="Rocket_Launcher_0001"; Version="1";
        #         Path="/acme"
        # 
        # and
        # 
        # Set-Cookie2: Part_Number="Riding_Rocket_0023"; Version="1";
        #         Path="/acme/ammo"

        interact(c, "http://www.acme.com/acme/ammo/specific",
                 'Part_Number="Rocket_Launcher_0001"; Version="1"; Path="/acme"',
                 'Part_Number="Riding_Rocket_0023"; Version="1"; Path="/acme/ammo"')

        # A subsequent request by the user agent to the (same) server for URLs of
        # the form /acme/ammo/...  would include the following request header:
        # 
        # Cookie: $Version="1";
        #         Part_Number="Riding_Rocket_0023"; $Path="/acme/ammo";
        #         Part_Number="Rocket_Launcher_0001"; $Path="/acme"
        # 
        # Note that the NAME=VALUE pair for the cookie with the more specific Path
        # attribute, /acme/ammo, comes before the one with the less specific Path
        # attribute, /acme.  Further note that the same cookie name appears more
        # than once.

        cookie = interact(c, "http://www.acme.com/acme/ammo/...")
        assert re.search(r"Riding_Rocket_0023.*Rocket_Launcher_0001", cookie)

        # A subsequent request by the user agent to the (same) server for a URL of
        # the form /acme/parts/ would include the following request header:
        # 
        # Cookie: $Version="1"; Part_Number="Rocket_Launcher_0001"; $Path="/acme"
        # 
        # Here, the second cookie's Path attribute /acme/ammo is not a prefix of
        # the request URL, /acme/parts/, so the cookie does not get forwarded to
        # the server.

        cookie = interact(c, "http://www.acme.com/acme/parts/")
        assert (string.find(cookie, "Rocket_Launcher_0001") != -1 and
                not string.find(cookie, "Riding_Rocket_0023") != -1)


    def test_6(self):
        """Test rejection of Set-Cookie2 responses based on domain, path, port.
        """
        from ClientCookie import Cookies

        c = Cookies()

        # illegal domain (no embedded dots)
        cookie = interact(c, "http://www.acme.com",
                          'foo=bar; domain=".com"; version=1')
        assert not c

        # legal domain
        cookie = interact(c, "http://www.acme.com",
                          'ping=pong; domain="acme.com"; version=1')
        assert len(c) == 1

        # illegal domain (host prefix "www.a" contains a dot)
        cookie = interact(c, "http://www.a.acme.com",
                          'whiz=bang; domain="acme.com"; version=1')
        assert len(c) == 1

        # legal domain
        cookie = interact(c, "http://www.a.acme.com",
                          'wow=flutter; domain=".a.acme.com"; version=1')
        assert len(c) == 2

        # can't partially match an IP-address
        cookie = interact(c, "http://125.125.125.125",
                          'zzzz=ping; domain="125.125.125"; version=1')
        assert len(c) == 2

        # illegal path (must be prefix of request path)
        cookie = interact(c, "http://www.sol.no",
                          'blah=rhubarb; domain=".sol.no"; path="/foo"; version=1')
        assert len(c) == 2

        # legal path
        cookie = interact(c, "http://www.sol.no/foo/bar",
                          'bing=bong; domain=".sol.no"; path="/foo"; version=1')
        assert len(c) == 3

        # illegal port (request-port not in list)
        cookie = interact(c, "http://www.sol.no",
                          'whiz=ffft; domain=".sol.no"; port="90,100"; version=1')
        assert len(c) == 3

        # legal port
        cookie = interact(c, "http://www.sol.no",
                          r'bang=wallop; version=1; domain=".sol.no"; '
                          r'port="90,100, 80,8080"; '
                          r'max-age=100; Comment = "Just kidding! (\"|\\\\) "')
        assert len(c) == 4

        # port attribute without any value (current port)
        cookie = interact(c, "http://www.sol.no",
                          'foo9=bar; version=1; domain=".sol.no"; port; max-age=100;')
        assert len(c) == 5

        # encoded path
        cookie = interact(c, "http://www.sol.no/foo/",
                          r'foo8=bar; version=1; path="/%66oo"')
        assert len(c) == 6

        # save and restore
        file = "lwp-cookies.txt"

        try:
            c.save(file)
            old = str(c)

            c = Cookies()
            c.load(file)
        finally:
            try: os.unlink(file)
            except IOError: pass

        assert old == str(c)


    def test_7(self):
        """Try some URL encodings of the PATHs."""
        from ClientCookie import Cookies

        c = Cookies()
        interact(c, "http://www.acme.com/foo%2f%25/%40%40%0Anew%E5/%E5",
                 "foo  =   bar; version    =   1")

        cookie = interact(c, "http://www.acme.com/foo%2f%25/@@%0anew/",
                          'bar=baz; path="/foo/"; version=1');
        version_re = re.compile(r'^\$version=\"?1\"?', re.I)
        assert (string.find(cookie, "foo=bar") != -1 and version_re.search(cookie))

        cookie = interact(c, "http://www.acme.com/foo/%25/@@%0anew/")
        assert not cookie


    def test_8(self):
        """Try to use the Netscape cookie file format for saving."""
        from ClientCookie import NetscapeCookies

        file = "cookies.txt"

        c = NetscapeCookies(file)
        interact(c, "http://www.acme.com/", "foo1=bar; max-age=100; Version=1")
        interact(c, "http://www.acme.com/",
                 'foo2=bar; port="80"; max-age=100; Discard; Version=1')
        interact(c, "http://www.acme.com/", "foo3=bar; secure; Version=1")

        try:
            c.save()
            c = NetscapeCookies(file)
        finally:
            try: os.unlink(file)
            except IOError: pass

        assert len(c) == 1  # 2 of them discarded on save
        assert string.find(str(c), "foo1=bar") != -1


    def test_9(self):
        """Some additional Netscape cookies test."""
        from ClientCookie import Cookies

        c = Cookies()
        headers = []
        req = urllib2.Request("http://foo.bar.acme.com/foo")

        # Netscape allows a host part that contains dots
        headers.append("Set-Cookie: Customer=WILE_E_COYOTE; domain=.acme.com")
        res = FakeResponse(headers, "http://www.acme.com/foo")
        c.extract_cookies(res, req)

        # and that the domain is the same as the host without adding a leading
        # dot to the domain.  Should not quote even if strange chars are used
        # in the cookie value.
        headers.append("Set-Cookie: PART_NUMBER=3,4; domain=foo.bar.acme.com")
        res = FakeResponse(headers, "http://www.acme.com/foo")
        c.extract_cookies(res, req)

        #print c

        req = urllib2.Request("http://foo.bar.acme.com/foo")
        c.add_cookie_header(req)
        assert (string.find(req.headers["Cookie"], "PART_NUMBER=3,4") != -1 and
                string.find(req.headers["Cookie"], "Customer=WILE_E_COYOTE") != -1)


    def test_9(self):
        """Test handling of local intranet hostnames without a dot."""
        from ClientCookie import Cookies

        c = Cookies()
        interact(c, "http://example/", "foo1=bar; PORT; Discard; Version=1;")
        cookie = interact(c, "http://example/", 'foo2=bar; domain=".local"; Version=1')
        assert not (string.find(cookie, "foo1=bar") == -1)

        interact(c, "http://example/", 'foo3=bar; Version=1')
        cookie = interact(c, "http://example/")
        assert not (string.find(cookie, "foo2=bar") == -1 or len(c) != 3)


    def test_10(self):
        from ClientCookie import Cookies

        # Test for empty path
        # Broken web-server ORION/1.3.38 returns to the client response like
        #
        #	Set-Cookie: JSESSIONID=ABCDERANDOM123; Path=
        #
        # ie. with Path set to nothing.
        # In this case routine extract_cookies() must set cookie to / (root)
        #print "---"
        #print "Test for empty path..."
        c = Cookies()  # clear it
        headers = []

        req = urllib2.Request("http://www.ants.com/")
        headers.append("Set-Cookie: JSESSIONID=ABCDERANDOM123; Path=")
        res = FakeResponse(headers, "http://www.ants.com/")
        #print res
        c.extract_cookies(res, req)
        #print c

        req = urllib2.Request("http://www.ants.com/")
        c.add_cookie_header(req)

        assert (req.headers["Cookie"] == "JSESSIONID=ABCDERANDOM123" and
                req.headers["Cookie2"] == '$Version="1"')


        # I don't understand what this test is for
        # missing path in the request URI
        req = urllib2.Request("http://www.ants.com:8080")
        c.add_cookie_header(req)

        assert (req.headers["Cookie"] == "JSESSIONID=ABCDERANDOM123" and
                req.headers["Cookie2"] == '$Version="1"')
        #print "skipping 34"


# The correctness of this test is undefined as of 2003-04-28, pending the
# RFC 2965 errata.
##     def test_11(self):
##         """test mixing of Set-Cookie and Set-Cookie2 headers."""
##         from ClientCookie import Cookies

##         # Example from http://www.trip.com/trs/trip/flighttracker/flight_tracker_home.xsl
##         # which gives up these headers:
##         #
##         # HTTP/1.1 200 OK
##         # Connection: close
##         # Date: Fri, 20 Jul 2001 19:54:58 GMT
##         # Server: Apache/1.3.19 (Unix) ApacheJServ/1.1.2
##         # Content-Type: text/html
##         # Content-Type: text/html; charset=iso-8859-1
##         # Link: </trip/stylesheet.css>; rel="stylesheet"; type="text/css"
##         # Servlet-Engine: Tomcat Web Server/3.2.1 (JSP 1.1; Servlet 2.2; Java 1.3.0; SunOS 5.8 sparc; java.vendor=Sun Microsystems Inc.)
##         # Set-Cookie: trip.appServer=1111-0000-x-024;Domain=.trip.com;Path=/
##         # Set-Cookie: JSESSIONID=fkumjm7nt1.JS24;Path=/trs
##         # Set-Cookie2: JSESSIONID=fkumjm7nt1.JS24;Version=1;Discard;Path="/trs"
##         # Title: TRIP.com Travel - FlightTRACKER
##         # X-Meta-Description: Trip.com privacy policy
##         # X-Meta-Keywords: privacy policy

##         req = urllib2.Request(
##             'http://www.trip.com/trs/trip/flighttracker/flight_tracker_home.xsl')
##         headers = []
##         headers.append("Set-Cookie: trip.appServer=1111-0000-x-024;Domain=.trip.com;Path=/")
##         headers.append("Set-Cookie: JSESSIONID=fkumjm7nt1.JS24;Path=/trs")
##         headers.append('Set-Cookie2: JSESSIONID=fkumjm7nt1.JS24;Version=1;Discard;Path="/trs"')
##         res = FakeResponse(
##             headers,
##             'http://www.trip.com/trs/trip/flighttracker/flight_tracker_home.xsl')
##         #print res

##         c = Cookies()
##         c.extract_cookies(res, req)
##         #print c
##         print str(c)
##         print """Set-Cookie3: trip.appServer="1111-0000-x-024"; path="/"; domain=".trip.com"; path_spec; discard; version=0
##         Set-Cookie3: JSESSIONID="fkumjm7nt1.JS24"; path="/trs"; domain="www.trip.com"; path_spec; discard; version=1
##         """
##         assert str(c) == """Set-Cookie3: trip.appServer="1111-0000-x-024"; path="/"; domain=".trip.com"; path_spec; discard; version=0
##         Set-Cookie3: JSESSIONID="fkumjm7nt1.JS24"; path="/trs"; domain="www.trip.com"; path_spec; discard; version=1
##         """


    def test_12(self):
        from ClientCookie import Cookies

        year_plus_one = localtime(time.time())[0] + 1

        # Test if temporary cookies are deleted properly with
        # jar.clear_temporary_cookies()

        req = urllib2.Request('http://www.perlmeister.com/scripts')
        headers = []
        headers.append("Set-Cookie: s1=session;Path=/scripts")
        headers.append("Set-Cookie: p1=perm; Domain=.perlmeister.com;"
                       "Path=/;expires=Fri, 02-Feb-%d 23:24:20 GMT" % year_plus_one)
        headers.append("Set-Cookie: p2=perm;Path=/;expires=Fri, "
                       "02-Feb-%d 23:24:20 GMT" % year_plus_one)
        headers.append("Set-Cookie: s2=session;Path=/scripts;Domain=.perlmeister.com")
        headers.append('Set-Cookie2: s3=session;Version=1;Discard;Path="/"')
        res = FakeResponse(headers, 'http://www.perlmeister.com/scripts')

        c = Cookies()  # clear jar
        c.extract_cookies(res, req)
        # How many session/permanent cookies do we have?
        counter = {"session_after": 0}
        def callback(args, counter=counter):
            key = "%s_before" % (args[2],)
            if not counter.has_key(key): counter[key] = 0
            counter[key] = counter[key] + 1
        c.scan(callback)
        c.clear_temporary_cookies()
        # How many now?
        def callback(args, counter=counter):
            key = "%s_after" % (args[2],)
            if not counter.has_key(key): counter[key] = 0
            counter[key] = counter[key] + 1
        c.scan(callback)
        assert not (
            # a permanent cookie got lost accidently
            counter["perm_after"] != counter["perm_before"] or
            # a session cookie hasn't been cleared
            counter["session_after"] != 0 or
            # we didn't have session cookies in the first place
            counter["session_before"] == 0)
